import jwt
from sanic import Sanic, response
from datetime import datetime, timedelta
from autobahn.asyncio.wamp import ApplicationRunner, ApplicationSession
import logging
import asyncio
import configargparse

parser = configargparse.get_argument_parser()
parser.add_argument("--http-port", type=int, default=8000)
parser.add_argument("--days", type=int, default=3, help="token validity in days")
parser.add_argument(
    "--secret-mev", default="zxcvbasdfg"
)  # coincide con secret di mev_cloud

log = logging.getLogger(__name__)


class Authorize(object):
    # baseurl = ''  # ridefinita da initialize

    def __init__(self, audience=""):
        self.audience = audience

    def __call__(self, fn):
        """
        chiamato una volta sola, durante la decorazione

        non è utilizzato response.redirect perché codifica url causando un problema
        """

        def new_func(*args, **kwargs):
            component = args[0]
            request = args[1]
            kwargs["payload"] = {}

            token = (
                request.token
                or request.args.get("token", "")
                or request.form.get("token", "")
            )

            # audience = self.audience or kwargs.get('audience') #or (component.settings.get(
            # 'jwt', {}).get('payload') or {}).get('aud', '')

            # print('secret:', component.settings.secret)
            # print('token:', token, type(token))
            try:
                payload = jwt.decode(
                    token,
                    component.settings.secret_mev,
                    # audience=audience,
                    algorithms=["HS256"],
                )
                # alcuni servizi generano l'url che permette di fare altre query
                payload["token"] = token
                kwargs["payload"] = payload
            except jwt.exceptions.ExpiredSignatureError:
                return response.json({"error": "token expired"}, status=400)
            except jwt.exceptions.InvalidAudienceError:
                return response.json({"error": "audience invalid"}, status=400)
            except jwt.exceptions.DecodeError:
                return response.json({"error": "token is not valid"}, status=400)
            except jwt.exceptions.MissingRequiredClaimError:
                return response.json(
                    {"error": "token has no correct audience"}, status=400
                )

            return fn(*args, **kwargs)

        return new_func


class RestApi:
    token_default = ""

    def __init__(self, config):
        self.settings = config

        app = Sanic("mev")

        app.add_route(self.testToken, "/test")
        app.add_route(self.testToken, "/")
        app.add_route(self.tokenGenerate, "/gen")
        # app.add_route(self.getContextImage, '/getContextImage/<key>')
        app.add_route(self.getImage, "/getImage/<encode>/<kind>/<key>")
        # app.add_route(self.getZones, '/getZones')

        srv_coro = app.create_server(
            host="0.0.0.0",
            port=self.settings.http_port,
            return_asyncio_server=True,
            asyncio_server_kwargs=dict(start_serving=False),
        )
        log.info(
            "starting rest http API on http://localhost:%s", self.settings.http_port
        )
        loop = asyncio.get_event_loop()
        self.srv = loop.run_until_complete(srv_coro)
        loop.run_until_complete(self.srv.startup())
        loop.run_until_complete(self.srv.start_serving())
        loop.create_task(self.srv.serve_forever())

    async def onJoin(self, details):
        # log.info("wamp session attached to realm %s", self.settings.realm)
        self.token_default = self.getToken()
        self.register(self.getToken, "token.get")

    async def onDisconnect(self):
        log.warning("wamp transport lost, shutting down")
        self.close()
        asyncio.get_event_loop().stop()

    def close(self):
        log.warning("closing ...")
        self.srv.before_stop()
        self.srv.close()

    @Authorize()
    def testToken(self, request, audience="", payload={}):
        return response.json(payload)

    def getToken(self, payload={}, default=True):
        if default and self.token_default:
            return self.token_default
        else:
            payload["exp"] = datetime.utcnow() + timedelta(days=self.settings.days)
            token = jwt.encode(payload, self.settings.secret_mev).decode("utf-8")

            return {
                "token": token,
                "exp": datetime.fromtimestamp(payload["exp"]).isoformat(),
                # 'aud': 'xxx',
            }

    async def tokenGenerate(self, request):
        return response.json(self.getToken())

    # async def getContextImage(self, request, audience='', payload={}, key=''):
    #     '''overload me please'''
    #     pass

    async def getImage(
        self, request, audience="", payload={}, key="", kind="context", encode="binary"
    ):
        """overload me please"""
        pass

    # async def getZones(self, request):
    #     '''overload me please'''
    #     return response.json(dict(xxx=key))
